'''Python script that produces reStructuredText out of JUnit/xUnit output (XML)'''
import argparse
import logging
import xml.etree.ElementTree as ET
from collections import namedtuple
from pathlib import Path

from mako.exceptions import RichTraceback
from mako.runtime import Context
from mako.template import Template
from pkg_resources import DistributionNotFound, require

TraceableInfo = namedtuple("TraceableInfo", ['matrix_prefix', 'unit_or_integration', 'header_prefix'])
UTEST = TraceableInfo('UTEST_', 'unit', '_unit_test_report_')
ITEST = TraceableInfo('ITEST_', 'integration', '_integration_test_report_')
TEMPLATE_FILE = Path(__file__).parent.joinpath('xunit2rst.mako')


def render_template(destination, **kwargs):
    """ Renders the Mako template, and writes output file to the specified destination.

    Args:
        destination (Path): Location of the output file.
        **kwargs (dict): Variables to be used in the Mako template.

    Raises:
        ERROR: Error is raised by Mako template.
    """
    destination.parent.mkdir(parents=True, exist_ok=True)
    with open(destination, 'w', newline='\n') as rst_file:
        template = Template(filename=str(TEMPLATE_FILE), output_encoding='utf-8', input_encoding='utf-8')
        try:
            template.render_context(Context(rst_file, **kwargs))
        except OSError:
            traceback = RichTraceback()
            for (filename, lineno, function, line) in traceback.traceback:
                logging.error("File %s, line %s, in %s", filename, lineno, function)
                logging.error(line, "\n")
            logging.error("%s: %s", str(traceback.error.__class__.__name__), traceback.error)


def generate_xunit_to_rst(input_file, rst_file, prefix):
    """ Calls mako template function and passes all needed parameters.

    Args:
        input_file (Path): Path to the input file (.xml).
        rst_file (Path): Path to the output file (.rst).
        prefix (str): Prefix to add to item IDs.
    """
    test_suites, prefix_set = parse_xunit_root(input_file)
    if not prefix and prefix_set is ITEST:
        prefix = ITEST.matrix_prefix.replace('_', '-')

    report_name = rst_file.stem
    if report_name.endswith('_report'):
        report_name = report_name[:-len('_report')]

    render_template(
        rst_file,
        test_suites=test_suites,
        rst_file=rst_file,
        report_name=report_name,
        info=prefix_set,
        prefix=prefix,
    )


def parse_xunit_root(input_file):
    '''
    This function parses the root element of the XML file and returns a testsuites root element and the set of prefixes
    to use.

    Args:
        input_file (str): Path to the input XML file.

    Returns:
        xml.etree.ElementTree.Element: root element with testsuites as tag
        namedtuple: Set of prefixes to use for building traceability output
    '''
    tree = ET.parse(input_file)
    root_input = tree.getroot()
    if root_input.tag != 'testsuites':
        test_suites = ET.Element("testsuites")
        test_suites.append(root_input)
        prefix_set = ITEST
    else:
        test_suites = root_input
        prefix_set = UTEST

    return test_suites, prefix_set


def main():
    '''Main function'''
    try:
        version = require('mlx.xunit2rst')[0].version
    except DistributionNotFound:
        from .__xunit2rst_version__ import version
    arg_parser = argparse.ArgumentParser()
    arg_parser.add_argument('-i', '--input',
                            action='store',
                            required=True,
                            dest='input_file',
                            type=Path,
                            help='The input XML file',)
    arg_parser.add_argument('-o', '--output',
                            action='store',
                            required=True,
                            dest='rst_output_file',
                            type=Path,
                            help='The output RST file',)
    arg_parser.add_argument('-p', '--prefix',
                            action='store',
                            default="",
                            help='Optional prefix to add to item IDs',)
    arg_parser.add_argument("--trim-suffix", action='store_true',
                            help="If the suffix of the --prefix argument ends with '_-' it gets trimmed to '-'")
    arg_parser.add_argument('-v', '--version',
                            action='version',
                            version='%(prog)s {}'.format(version),)
    args = arg_parser.parse_args()

    prefix = args.prefix
    if prefix.endswith('_-') and args.trim_suffix:
        prefix = prefix.rstrip('_-') + '-'

    generate_xunit_to_rst(args.input_file, args.rst_output_file, prefix)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
''' Acceptance test suite for mlx.xunit2rst

The tests in this file are pure black box tests. They just check whether the
outputs of the tool match the reference output files exactly.
'''

from subprocess import check_call
import filecmp
from pathlib import Path

import nose
from nose.tools import with_setup

TOP_DIR = Path(__file__).parents[1]
TEST_OUT_DIR = Path(__file__).parent / 'test_out'
TEST_IN_DIR = Path(__file__).parent / 'test_in'


def setup():
    ''' Setup function

    This function creates the output directory for mlx.xunit2rst to put the
    output files
    '''
    TEST_OUT_DIR.mkdir(exist_ok=True)


def xunit2rst_check(input_xml, output_rst, prefix='', trim=False):
    ''' Helper function for testing whether mlx.xunit2rst produces the expected output '''

    command = 'mlx.xunit2rst -i {} -o {}'.format(input_xml, output_rst)
    if prefix:
        command += ' -p {}'.format(prefix)
    if trim:
        command += ' --trim-suffix'

    return_code = check_call(command, shell=True)
    print(command)
    assert return_code == 0


@with_setup(setup)
def test_xunit_default():
    '''Tests based on reports generated by `robot --xunit` '''
    file_name = 'itest_report'
    rst_file_name = '{}.rst'.format(file_name)
    xml_file_name = '{}.xml'.format(file_name)
    input_xml = str(TEST_IN_DIR / xml_file_name)
    output_rst = str(TEST_OUT_DIR / rst_file_name)
    xunit2rst_check(input_xml, output_rst, prefix='ITEST-')

    reference_rst = str(TEST_IN_DIR / rst_file_name)
    assert filecmp.cmp(output_rst, reference_rst)


@with_setup(setup)
def test_xunit_custom_prefix():
    '''Tests based on reports generated by `robot --xunit` '''
    file_name = 'itest_lin_report'
    rst_file_name = '{}.rst'.format(file_name)
    xml_file_name = '{}.xml'.format(file_name)
    input_xml = str(TEST_IN_DIR / xml_file_name)
    output_rst = str(TEST_OUT_DIR / rst_file_name)
    xunit2rst_check(input_xml, output_rst, prefix='ITEST_LIN-')

    reference_rst = str(TEST_IN_DIR / rst_file_name)
    assert filecmp.cmp(output_rst, reference_rst)


@with_setup(setup)
def test_xunit_no_prefix():
    '''Tests based on reports generated by `robot --xunit` '''
    file_name = 'itest_report'
    rst_file_name = '{}.rst'.format(file_name)
    xml_file_name = '{}.xml'.format(file_name)
    input_xml = str(TEST_IN_DIR / xml_file_name)
    output_rst = str(TEST_OUT_DIR / rst_file_name)
    xunit2rst_check(input_xml, output_rst)

    reference_rst = str(TEST_IN_DIR / rst_file_name)
    assert filecmp.cmp(output_rst, reference_rst)


@with_setup(setup)
def test_xunit_trim_suffix():
    '''Tests based on reports generated by `robot --xunit` '''
    file_name = 'itest_lin_report'
    rst_file_name = '{}.rst'.format(file_name)
    xml_file_name = '{}.xml'.format(file_name)
    input_xml = str(TEST_IN_DIR / xml_file_name)
    output_rst = str(TEST_OUT_DIR / rst_file_name)
    xunit2rst_check(input_xml, output_rst, prefix='ITEST_LIN_-', trim=True)

    reference_rst = str(TEST_IN_DIR / rst_file_name)
    assert filecmp.cmp(output_rst, reference_rst)


@with_setup(setup)
def test_xunit_trim_suffix_not_needed():
    '''Tests based on reports generated by `robot --xunit` '''
    file_name = 'itest_report'
    rst_file_name = '{}.rst'.format(file_name)
    xml_file_name = '{}.xml'.format(file_name)
    input_xml = str(TEST_IN_DIR / xml_file_name)
    output_rst = str(TEST_OUT_DIR / rst_file_name)
    xunit2rst_check(input_xml, output_rst, prefix='ITEST-', trim=True)

    reference_rst = str(TEST_IN_DIR / rst_file_name)
    assert filecmp.cmp(output_rst, reference_rst)


@with_setup(setup)
def test_junit():
    '''Tests based on utest reports in JUnit format '''
    file_name = 'utest_my_lib_report'
    rst_file_name = '{}.rst'.format(file_name)
    xml_file_name = '{}.xml'.format(file_name)
    input_xml = str(TEST_IN_DIR / xml_file_name)
    output_rst = str(TEST_OUT_DIR / rst_file_name)
    xunit2rst_check(input_xml, output_rst)

    reference_rst = str(TEST_IN_DIR / rst_file_name)
    assert filecmp.cmp(output_rst, reference_rst)


if __name__ == '__main__':
    nose.main()



from setuptools import setup, find_packages

requires = ['mako']

setup(
    name='mlx.xunit2rst',
    use_scm_version={
        'write_to': 'mlx/__xunit2rst_version__.py'
    },
    setup_requires=['setuptools_scm'],
    url='https://github.com/melexis/xunit2rst',
    license='Apache License Version 2.0',
    author='JasperCraeghs',
    author_email='jce@melexis.com',
    description='Python script for converting xUnit/JUnit XML format to reStructuredText format',
    long_description=open("README.rst").read(),
    long_description_content_type='text/x-rst',
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Documentation',
        'Topic :: Utilities',
    ],
    platforms='any',
    packages=find_packages(exclude=['tests', 'doc']),
    package_data={'mlx.xunit2rst': ['mlx/*.mako']},
    include_package_data=True,
    install_requires=requires,
    namespace_packages=['mlx'],
    keywords=['xUnit', 'JUnit', 'XML', 'reStructuredText', 'sphinx', 'rst', 'testing', 'traceability', 'documentation'],
    entry_points={
        'console_scripts': ['mlx.xunit2rst = mlx.xunit2rst:main']
    },
)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import os
import mlx.traceability
from pkg_resources import get_distribution
from pygments.formatters import LatexFormatter

# -- Path setup --------------------------------------------------------------
# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
# sys.path.insert(0, os.path.abspath('../mlx'))

# -- Project information -----------------------------------------------------

project = 'mlx.xunit2rst'
copyright = '2019, Bavo Van Achte'
authors = ['Bavo Van Achte', 'Jasper Craeghs']

# The full version, including alpha/beta/rc tags
release = get_distribution('mlx.xunit2rst').version
version = '.'.join(release.split('.')[:2])

latex_documents = [
    ('index', 'xunit2rst.tex', 'Script to convert .robot files to .rst files with traceable items',
     ' \\and '.join(authors), 'manual', True),
]

man_pages = [
    ('index', 'xunit2rst', 'Script to convert .robot files to .rst files with traceable items',
     authors, 1)
]

# -- Options for Texinfo output ------------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    ('index', 'xunit2rst', 'xunit2rst conversion script', '@*'.join(authors), 'xunit2rst',
     'Script to convert .robot files to .rst files with traceable items.', 'Miscellaneous'),
]

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinxcontrib_robotdoc',
    'mlx.traceability',
]

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    'preamble': r'\usepackage{fancyvrb}'
                r'\usepackage{color}'
                + LatexFormatter().get_style_defs(),
}

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Additional attributes for traceability plugin
traceability_relationships = {
    'passes': 'passed_by',
    'fails': 'failed_by',
    'validates': 'validated_by',
}
traceability_relationship_to_string = {
    'passes': 'Passes',
    'passed_by': 'Passed by',
    'fails': 'Fails',
    'failed_by': 'Failed by',
    'validates': 'Validates',
    'validated_by': 'Validated by',
}
# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = [os.path.join(os.path.dirname(mlx.traceability.__file__), 'assets')]

traceability_render_relationship_per_item = True

def setup(app):
    pass
